@@ -3,7 +3,7 @@
 // Rcpp.h: R/C++ interface class library
 //
 // Copyright (C) 2008 - 2009 Dirk Eddelbuettel
-// Copyright (C) 2009 - 2012 Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2009 - 2014 Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -26,31 +26,30 @@
 /* it is important that this comes first */
 #include <RcppCommon.h>
 
-/* new api */
-#include <Rcpp/exceptions.h>
-
 #include <Rcpp/RObject.h>
 
-#include <Rcpp/Promise.h>
 #include <Rcpp/S4.h>
 #include <Rcpp/Reference.h>
 #include <Rcpp/clone.h>
 #include <Rcpp/grow.h>
 #include <Rcpp/Dimension.h>
 #include <Rcpp/Environment.h>
-#include <Rcpp/Evaluator.h>
 
 #include <Rcpp/Vector.h>
 #include <Rcpp/sugar/nona/nona.h>
 #include <Rcpp/Fast.h>
 #include <Rcpp/Extractor.h>
+#include <Rcpp/Promise.h>
 
 #include <Rcpp/XPtr.h>
 #include <Rcpp/Symbol.h>
+#include <Rcpp/DottedPairImpl.h>
+#include <Rcpp/Function.h>
 #include <Rcpp/Language.h>
 #include <Rcpp/DottedPair.h>
 #include <Rcpp/Pairlist.h>
-#include <Rcpp/Function.h>
+#include <Rcpp/StretchyList.h>
+
 #include <Rcpp/WeakReference.h>
 #include <Rcpp/StringTransformer.h>
 #include <Rcpp/Formula.h>
@@ -60,12 +59,11 @@
 #include <Rcpp/Datetime.h>
 #include <Rcpp/DatetimeVector.h>
 
+#include <Rcpp/Na_Proxy.h>
+
 #include <Rcpp/Module.h>
 #include <Rcpp/InternalFunction.h>
 
-#include <Rmath.h>
-#include <Rcpp/sugar/undoRmath.h>
-
 #ifndef RCPP_NO_SUGAR
 #include <Rcpp/sugar/sugar.h>
 #include <Rcpp/stats/stats.h>
@@ -78,6 +76,6 @@
 // 'see' all versions of wrap
 #include <Rcpp/internal/wrap_end.h>
 
+#include <Rcpp/platform/solaris.h>
 #include <Rcpp/api/meat/meat.h>
-
 #endif
