@@ -51,6 +51,11 @@ struct wrap_type_module_object_tag{} ;
  */
 struct wrap_type_enum_tag{} ;
 
+/**
+ * character arrays
+ */
+struct wrap_type_char_array{} ;
+
 /**
  * Type trait that helps the dispatch of wrap to the proper method
  *
@@ -62,6 +67,8 @@ struct wrap_type_enum_tag{} ;
  */
 template <typename T> struct wrap_type_traits { typedef wrap_type_unknown_tag wrap_category; } ;
 
+template <std::size_t N> struct wrap_type_traits<char[N]> { typedef wrap_type_char_array wrap_category; } ;
+template <std::size_t N> struct wrap_type_traits<const char[N]> { typedef wrap_type_char_array wrap_category; } ;
 /**
  * Total specialization for primitive types
  */
