@@ -147,7 +147,13 @@ namespace Rcpp{
 	namespace internal{
 		template <typename Class> SEXP make_new_object( Class* ptr ) ;	
 	}
-}	
+}
+
+#include <Rcpp/longlong.h>
+
+#include <Rcpp/internal/na.h>
+#include <Rcpp/internal/NAComparator.h>
+#include <Rcpp/internal/NAEquals.h>
 
 #include <Rcpp/traits/traits.h>
 #include <Rcpp/Named.h>
@@ -167,8 +173,6 @@ namespace Rcpp{
 #include <Rcpp/vector/VectorBase.h>
 #include <Rcpp/vector/MatrixBase.h>
 
-#include <Rcpp/internal/wrap.h>
-
 #include <Rcpp/internal/ListInitialization.h>
 #include <Rcpp/internal/Proxy_Iterator.h>
 #include <Rcpp/internal/SEXP_Iterator.h>
@@ -179,6 +183,7 @@ namespace Rcpp{
 #include <Rcpp/sugar/sugar_forward.h>
 
 #include <Rcpp/iostream/Rstreambuf.h>
-#include <Rcpp/longlong.h>
+
+#include <Rcpp/internal/wrap.h>
 
 #endif
