@@ -186,13 +186,6 @@ int integer_names_indexing( IntegerVector y ){
     return y["foo"] ;
 }
 
-// [[Rcpp::export]]
-IntegerVector integer_comma(){
-    IntegerVector x(4) ;
-	x = 0, 1, 2, 3 ;
-	return x ;
-}
-
 // [[Rcpp::export]]
 IntegerVector integer_push_back( IntegerVector y ){
     y.push_back( 5 ) ;
@@ -599,13 +592,6 @@ std::string character_names_indexing( CharacterVector y ){
     return foo ;
 }
 
-// [[Rcpp::export]]
-CharacterVector character_comma(){
-    CharacterVector x(3) ;
-    x = "foo", "bar", "bling" ;
-    return x ;
-}
-
 // [[Rcpp::export]]
 List character_listOf( List ll ){
     CharacterVector cv1 = ll["foo"];
@@ -681,54 +667,6 @@ List List_rep_ctor(IntegerVector x){
     return List(3, x) ;
 }
 
-#if defined(HAS_INIT_LISTS)
-    RawVector raw_initializer_list(){
-        RawVector x = {0,1,2,3} ;
-        for( int i=0; i<x.size(); i++) x[i] = x[i]*2 ;
-        return x ;
-    }
-    
-    ComplexVector complex_initializer_list(){
-        Rcomplex c1 ; c1.r = c1.i = 0.0 ;
-    	Rcomplex c2 ; c2.r = c2.i = 1.0 ;
-    	ComplexVector x = { c1, c2 } ;
-    	return x ;
-    }
-    
-    IntegerVector integer_initializer_list(){
-        IntegerVector x = {0,1,2,3} ;
-    	for( int i=0; i<x.size(); i++) x[i] = x[i]*2 ;
-    	return x ;
-    }
-    
-    NumericVector numeric_initlist(){
-        NumericVector x = {0.0,1.0,2.0,3.0} ;
-    	for( int i=0; i<x.size(); i++) x[i] = x[i]*2 ;
-    	return x ;
-    }
-    
-    List list_initializer_list(){
-        SEXP x0 = PROTECT( Rf_ScalarInteger( 0 ) ) ;
-        SEXP x1 = PROTECT( Rf_ScalarInteger( 1 ) ) ;
-        SEXP x2 = PROTECT( Rf_ScalarInteger( 2 ) ) ;
-        List x = { x0, x1, x2} ;
-        UNPROTECT(3) ;
-        return x ;
-    }
-    
-    CharacterVector character_initializer_list(){
-        CharacterVector x = {"foo", "bar"} ;
-        return x ;
-    }
-    
-    List List_insert(){
-        List list(x) ;
-        list.insert( list.begin(), 10 ) ;
-        list.insert( list.end(), Named("foo", "bar" ) ) ;
-        return list ;
-	}
-#endif
-
 // [[Rcpp::export]]
 int stdVectorDouble(std::vector<double> x) { 
     return x.size();
@@ -768,3 +706,47 @@ int stdVectorIntRef(std::vector<int> & x) {
 int stdVectorIntConstRef(const std::vector<int> & x) { 
     return x.size();
 }
+
+// [[Rcpp::export]]
+std::string character_vector_const_proxy(const CharacterVector& str){
+    char* cstr = (char*) str[0] ;
+    std::string res  ;
+    res += cstr ;
+    return cstr ;
+}
+
+// [[Rcpp::export]]
+CharacterVector CharacterVector_test_const_proxy(const CharacterVector x){
+    CharacterVector out( x.size() ) ;
+    for( int i=0; i<x.size(); i++){
+        out[i] = x[i] ;    
+    }
+    return out ;
+}
+
+// [[Rcpp::export]]
+NumericVector sort_numeric(NumericVector x) {
+  return x.sort();
+}
+
+// [[Rcpp::export]]
+IntegerVector sort_integer(IntegerVector x) {
+  return x.sort();
+}
+
+// [[Rcpp::export]]
+CharacterVector sort_character(CharacterVector x) {
+  return x.sort();
+}
+
+// [[Rcpp::export]]
+LogicalVector sort_logical(LogicalVector x) {
+  return x.sort();
+}
+
+// [[Rcpp::export]]
+List list_sexp_assign(SEXP x) {
+    List L;
+    L = x;
+    return L;
+}
